public class Reduction {

    private static final String IDENT_PATTERN = 
            "Multithreaded example with reduction and critical section\n";
    private static final String INIT_DATA_PATTERN = "Initializing data ";
    private static final String INIT_THREADS_PATTERN = 
            "Initializing %d threads ";
    private static final String TIME_PATTERN = "(%d ms)\n";
    private static final String PROCESS_PATTERN = "Processing data\n";
    private static final String RESULT_PATTERN = 
            "\tCount = %d\n\tArithmetic mean = %f\n\tGeometric mean = %f\n";
    private static final int ARRAY_SIZE = 100000000;
    private static final int RANDOM_SEED = 20110523;
    private static final int NUM_THREADS = 4;
    
    private byte[] values;
    private int count;
    private long sum;
    private double logSum;
    private Worker[] workers;
    
    public static void main(String[] args) {
        Reduction manager = new Reduction();
        System.out.printf(IDENT_PATTERN);
        manager.initializeData();
        manager.initializeThreads();
        manager.process();
    }

    protected void initializeData() {
        long start = System.currentTimeMillis();
        java.util.Random rng = new java.util.Random(RANDOM_SEED);
        System.out.printf(INIT_DATA_PATTERN);
        values = new byte[ARRAY_SIZE];
        for (int i = 0; i < values.length; i++) {
            values[i] = (byte) (1 + rng.nextInt(Byte.MAX_VALUE));
        }
        count = 0;
        sum = 0;
        logSum = 0;
        System.out.printf(TIME_PATTERN, System.currentTimeMillis() - start);
    }
    
    protected void process() {
        long start = System.currentTimeMillis();
        System.out.printf(PROCESS_PATTERN);
        processWithThreads();
        System.out.printf(RESULT_PATTERN, 
                count, sum / (double) count, Math.exp(logSum / count));
        System.out.printf(TIME_PATTERN, System.currentTimeMillis() - start);
    }
    
    private void initializeThreads() {
        long start = System.currentTimeMillis();
        int sliceLength = values.length / NUM_THREADS;
        System.out.printf(INIT_THREADS_PATTERN, NUM_THREADS);
        workers = new Worker[NUM_THREADS];
        for (int i = 0; i < workers.length; i++) {
            workers[i] = new Worker(sliceLength * i, sliceLength);
        }
        System.out.printf(TIME_PATTERN, System.currentTimeMillis() - start);
    }
    
    protected void processWithThreads() {
        for (int i = 0; i < workers.length; i++) {
            workers[i].start();
        }
        for (int i = 0; i < workers.length; i++) {
            try {
                workers[i].join();
            }
            catch (InterruptedException e) {}
        }
    }
    
    public synchronized void update(int count, long value, double logValue) {
        sum += value;
        logSum += logValue;
        this.count += count;
    }
    
    class Worker extends Thread {

        private int rangeStart;
        private int rangeLength;

        public Worker(int rangeStart, int rangeLength) {
            this.rangeStart = rangeStart;
            this.rangeLength = rangeLength;
        }

        public void run() {
            long sum = 0;
            double logSum = 0;
            for (int i = rangeStart; i < rangeStart + rangeLength; i++) {
                sum += values[i];
                logSum += Math.log(values[i]);
            }
            update(rangeLength, sum, logSum);
        }

    }

}
