public class SingleThread {

    private static final String IDENT_PATTERN = "Single-threaded example\n";    
    private static final String INIT_DATA_PATTERN = "Initializing data ";
    private static final String TIME_PATTERN = "(%d ms)\n";
    private static final String PROCESS_PATTERN = "Processing data\n";
    private static final String RESULT_PATTERN = 
            "\tCount = %d\n\tArithmetic mean = %f\n\tGeometric mean = %f\n";
    private static final int ARRAY_SIZE = 100000000;
    private static final int RANDOM_SEED = 20110523;
    
    private byte[] values;
    private int count;
    private long sum;
    private double logSum;
    
    public static void main(String[] args) {
        SingleThread manager = new SingleThread();
        System.out.printf(IDENT_PATTERN);
        manager.initializeData();
        manager.process();
    }

    private void initializeData() {
        long start = System.currentTimeMillis();
        java.util.Random rng = new java.util.Random(RANDOM_SEED);
        System.out.printf(INIT_DATA_PATTERN);
        values = new byte[ARRAY_SIZE];
        for (int i = 0; i < values.length; i++) {
            values[i] = (byte) (1 + rng.nextInt(Byte.MAX_VALUE));
        }
        count = 0;
        sum = 0;
        logSum = 0;
        System.out.printf(TIME_PATTERN, System.currentTimeMillis() - start);
    }
    
    private void process() {
        long start = System.currentTimeMillis();
        System.out.printf(PROCESS_PATTERN);
        processWithoutThreads();
        System.out.printf(RESULT_PATTERN, 
                count, sum / (double) count, Math.exp(logSum / count));
        System.out.printf(TIME_PATTERN, System.currentTimeMillis() - start);
    }

    private void processWithoutThreads() {
        for (int i = 0; i < values.length; i++) {
            update(values[i]);
        }
    }
    
    public void update(byte value) {
        sum += value;
        logSum += Math.log(value);
        count++;
    }
    
}
